# nite-cropper
### Basic
<p>Consider a box <em>div.inscribe</em> which takes up a certain amount of space <em>(for wathever reason: width, height, padding...)</em>.</p>

```
<div class="inscribe">
   <img src="http://placehold.it/120x120.jpg" />
</div>
```

<p>If you want to <strong>cover ✂</strong> that element with an <em>&lt;img /&gt;</em> tag without using <em>css background properties</em>, just do:</p>

```
$('.inscribe').niteCrop();
```

<p>That's it! 👏 </p>

<hr>
<p>This plugin listens to images 'load', and videos 'onloadedmetadata' events so you don't need to wait for those stuff with other greedy scripts. 👌 </p>
<hr>

<p>You may need to address specific or deep-nested elements:</p>

```
$('.inscribe').niteCrop({
  elements : 'img.inscribe__element, video.inscribe__element'
});
```

<br>

### Advanced
<p>If you want to crop a block-level element (<em>&lt;div&gt;</em>,<em>&lt;span&gt;</em>, 🍫 etc...) you need to define its size like that:</p>

```
$('.inscribe').niteCrop({
  elements : 'div',
  width : 1920,
  height : 1240
});
```

<hr>

<p>🎟️ There's a dedicated <strong>event</strong> that occurs when the operation is done; here's some ways to do the same thing using that handler:</p>

```
$(document).on('crop.nite', function(event, $element, $target){

   $target.addClass('inscribed');
   
});
$('.inscribe').on('crop.nite', function(event, $target){

   $target.addClass('inscribed');
   
});
$(document).on('crop.nite', '.inscribe', function(event, $target){

   $target.addClass('inscribed');
   
});
$('.inscribe img.target').on('crop.nite', function(event, $element){

   $(this).addClass('inscribed');
   
});
$(document).on('crop.nite', '.inscribe img.target', function(event, $element){

   $(this).addClass('inscribed');
   
});

```

<hr>

<p>You programmatically changed your layout? You may need to refresh the crop calculation then...</p>

```
$('.inscribe').niteCrop('resize');
```

<hr>

<p>If you change your mind you can destroy an instance.</p>

```
$('.inscribe').niteCrop('destroy');
```

<hr>

<p>You can get useful data 🎒 with this:</p>

```
var usefulCropData = $('.inscribe img').data('niteCropper');
```
<br>

### Extra

<p>There's an awesome built-in parallax func! 😎</p>

```
$('.inscribe-with-parallax').niteCrop({
   parallax : true
});

$('.inscribe-with-parallax--horizontal-only').niteCrop({
   parallax : 'horizontal'
});

$('.inscribe-with-parallax--vertical-only').niteCrop({
   parallax : 'vertical'
});
```
<hr>

<br>

### Performance ⚡⚡⚡

<p>This plugin was designed to avoid layout <em>reflows</em>, <em>repaints</em> and other client-side <em> bottlenecks</em>, but some tasks, expecially <em>onScroll</em> ones, are usually quite demanding for browsers: if you plan to make use of 'parallax' option or you simply want to boost nite plugins performances up, you should add the mother-library <strong>"nite.js"</strong> as a dependency of your code. 👍</p>
