// JQuery Nite Cropper by memob0x ~ https://github.com/memob0x/nite-cropper/
// MIT license
(function (window, document, $, undefined){
    'use strict';


    // https://github.com/paulmillr/console-polyfill
    // - - - - - - - - - - - - - - - - - - - -
    (function(global) {
        if (!global.console)
            global.console = {};
        let con = global.console,
            prop, method,
            dummy = function() {},
            properties = ['memory'],
            methods = ('assert,clear,count,debug,dir,dirxml,error,exception,group,' +
                'groupCollapsed,groupEnd,info,log,markTimeline,profile,profiles,profileEnd,' +
                'show,table,time,timeEnd,timeline,timelineEnd,timeStamp,trace,warn').split(',');
        while (prop = properties.pop())
            if (!con[prop])
                con[prop] = {};
        while (method = methods.pop())
            if (!con[method])
                con[method] = dummy;
    })(window);
    // - - - - - - - - - - - - - - - - - - - -


    // niteCropper core crop logic
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
    const nite_crop_core = function( element, parent, scale ){

            if( typeof scale === 'undefined' )
                scale = 1;

            if( !( 'width' in element && 'height' in element && 'width' in parent && 'height' in parent ) ){

                console.error('inconsistent data provided to niteCrop function, can\'t proceed');

                return false;

            }

            let returned_data = {
                    width  : null,
                    height : null,
                    side   : null,
                    indentation :  {
                        x : 0,
                        y : 0
                    }
                },
                css = {
                    dim  : [ 'width', 'height' ],
                    coor : [ 'x', 'y' ]
                };

            for( let x in css.dim ) {

                let x_ = parseInt( x ),
                    _x = x_ === 1 ? 0 : 1;

                let side_a = css.dim [  x_],
                    side_b = css.dim [ _x ],
                    axis_a = css.coor[  x_],
                    axis_b = css.coor[ _x ];

                returned_data[ side_a ] = Math.round( parent[ side_a ] );
                returned_data[ side_b ] = Math.round( element[ side_b ] * ( parent[ side_a ] / element[ side_a ] ) );
                returned_data['side'] = side_a;
                returned_data['indentation'][ axis_a ] = 0;
                returned_data['indentation'][ axis_b ] = ( returned_data[ side_b ] - parent[ side_b ] ) / 2 / scale * - 1;

                if ( returned_data[ side_b ] >= parent[ side_b ] )
                    break;

            }

            return returned_data;

        };
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~

    // crop keygen globally available (Vanilla)
    window.niteCrop = nite_crop_core;
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~


    // jQuery is needed
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
    if( !$ ) {

        console.error('jQuery is needed for $.fn.niteCrop() to work!');

        return undefined;

    }
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~

    // crop keygen globally available (jQuery)
    $.niteCrop = niteCrop;
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~

    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
    let MutationObserver = (function () {
        let prefixes = ['WebKit', 'Moz', 'O', 'Ms', ''];
        for(let i=0; i < prefixes.length; i++)
            if( prefixes[i] + 'MutationObserver' in window )
                return window[prefixes[i] + 'MutationObserver'];
        return false;
    }());
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~

    // todo let const etc
    let
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
        plugin_name = 'niteCropper',
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
        $window = $(window),
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
        window_dimensions = {
            width  : null,
            height : null
        },
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
        matrix = function($el){

            if( $el.nodeType )
                $el = $($el);

            let backup = $el.css('transform');

            $el.css({
                transform : ''
            });

            let el_mx = $el.css('transform');

            // no transforms
            if( el_mx === 'none' )
                return false;

            $el.css({
                transform : backup
            });

            el_mx = el_mx
                .replace(/3d/g, '')
                .match(/-?[\d.]+/g);

            // something wrong / unexpected
            if( !el_mx || el_mx < 6 )
                return false;

            // NB
            // transform.length > 6 =  msie
            // transform.length === 6 = normal browsers

            return el_mx;


        },
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
        scale = function($el, el_mx){

            if( $el.nodeType )
                $el = $($el);

            el_mx = undefined === el_mx ? matrix($el) : el_mx;

            if( !el_mx )
                return 1;

            let a = el_mx[0],
                b = el_mx[1];

            return parseFloat( Math.sqrt( a * a + b * b ) );

        },

        build_transform_property = function(transform_data, hardware_acceleration){
            transform_data = $.extend(true, {
                scale : 0,
                rotate : 0,
                translate : [0, 0, 0]
            }, transform_data);
            return  ( transform_data['scale'] !== 1 ? 'scale('+ transform_data['scale'] +') ' : '' )+
                    ( transform_data['rotate'] !== 0 ? 'rotate('+ transform_data['rotate'] +'deg) ' : '' )+
                    (
                        true !== hardware_acceleration ?
                            ( transform_data['translate'][0] !== 0 ? 'translateX(' + transform_data['translate'][0] + 'px) ' : '' )+
                            ( transform_data['translate'][1] !== 0 ? 'translateY(' + transform_data['translate'][1] + 'px)' : '' )
                            :
                            'translate3d('+transform_data['translate'][0]+'px, '+transform_data['translate'][1]+'px, '+transform_data['translate'][2]+'px)'
                    );
        },

    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
        round_decimals = function(num, decs){
            return parseFloat(num.toFixed(decs));
        },
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
        rotation = function($el, el_mx){

            if( $el.nodeType )
                $el = $($el);

            el_mx = undefined === el_mx ? matrix($el) : el_mx;

            if( !el_mx )
                return 0;

            let a = el_mx[0],
                b = el_mx[1];

            return Math.round( Math.atan2(b, a) * ( 180 / Math.PI ) );

        };
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~


    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
    $.fn.niteCrop = function(options) {

        return this.each(function(){

            let
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                settings = {
                    elements    : null, //
                    width       : null, //
                    height      : null, //
                    parallax    : false, // false|true|'horizontal'|'x'|'vertical'|'y'
                    mutationObserverAttributes : [] //
                },
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                instance = null, // will be shortly defined...
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                $element_candidate = $(this),
                $element = $element_candidate.is('img') || $element_candidate.is('video') ? $element_candidate.parent() : $element_candidate,
                $elements = $(), // will be shortly defined...
                extra_observe_attrs = [],
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                is_parallax = function( strict ){

                    let is_everything = settings.parallax === true;

                    if( is_everything )
                        return is_everything;

                    let is_x = settings.parallax === 'x' || settings.parallax === 'horizontal',
                        is_y = settings.parallax === 'y' || settings.parallax === 'vertical';

                    if( undefined === strict )
                        return is_everything || is_x || is_y;

                    if( 'x' === strict )
                        return is_everything || is_x;

                    if( 'y' === strict )
                        return is_everything || is_y;

                },
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                do_indent = new (function(){

                    let indent_core = function( parent, child, coor, a,b,c, force ){

                        let decs = 6,
                            indentation = child.indentation[coor],
                            window_dimension = window_dimensions[c],
                            parent_dimension = parent[c],
                            child_dimension = child[c],
                            is_in_viewport = $.nite ? $.nite.inViewport($element[0]).ratio > 0 : parent.bottom >= 0 && parent.right >= 0 && parent.left <= window_dimensions.width && parent.top <= window_dimensions.height;

                        if ( true === force || ( is_in_viewport && child_dimension > parent_dimension ) ) {

                            let depth = undefined === $element.data('nite-cropper-parallax-depth') || !$.isNumeric( $element.data('nite-cropper-parallax-depth') ) ? 0.8 : parseFloat( $element.data('nite-cropper-parallax-depth') );
                            if( depth < 0.1 )
                                depth = 0.1;
                            if( depth > 1 )
                                depth = 1;

                            let win_diff = window_dimension - parent[a],

                                is_rev = win_diff > parent[b],
                                use_default_indentation = child.side !== c && indentation !== 0,

                                position_00_10_00 = round_decimals( /**/  ( is_rev ? parent[b] : win_diff ) / ( parent_dimension + ( window_dimension - parent_dimension ) / 2 ),  /**/ decs),
                                position_10_00_10 = round_decimals( /**/  1 - position_00_10_00,  /**/ decs),
                                position_00_05_10 = round_decimals( /**/  ( is_rev ? 1 + position_10_00_10 : position_00_10_00 ) / 2,  /**/ decs),
                                position_10_05_00 = round_decimals( /**/  1 - position_00_05_10,  /**/ decs),

                                parallax_ratio    = use_default_indentation ? position_10_05_00 * 2 : position_10_00_10 * ( is_rev ? - 1 : 1 );

                            indentation = use_default_indentation ? indentation : ( parent_dimension - child_dimension ) / 2;

                            depth = ( indentation * position_10_00_10 ) * ( is_rev ? 1 : - 1 ) * ( 1 - depth );

                            indentation = Math.round( indentation * parallax_ratio + depth );

                        }

                        return indentation;

                    };

                    this.x = function( parent, child, force ){

                        if( !is_parallax('x') )
                            return child.indentation.x;

                        return indent_core(parent, child, 'x', 'left', 'right', 'width', force);

                    };

                    this.y = function( parent, child, force ) {

                        if( !is_parallax('y') )
                            return child.indentation.y;

                        return indent_core(parent, child, 'y', 'top', 'bottom', 'height', force);

                    };

                })(),
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                scroll_handler = function(){

                    if( !is_parallax() )
                        return false;

                    $elements.each(function() {

                        let $t = $(this),
                            child = $t.data(plugin_name);

                        if( undefined === child )
                            return true;

                        child = $.extend(true, child, $.nite ? $.nite.getBoundingRects(this) : this.getBoundingClientRect());

                        let parent = $.extend(true, {}, $.nite ? $.nite.getBoundingRects($element[0]) : $element[0].getBoundingClientRect());

                        $t.css({ transform : build_transform_property({
                            scale : child.scale,
                            rotate : child.rotate,
                            translate : [
                                do_indent.x(parent, child),
                                do_indent.y(parent, child)
                            ]
                        }, is_parallax())});

                    });

                },


            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                initialized = undefined !== $element.data(plugin_name),
                initialize = function(event){

                    if( !$element.length )
                        return;

                    let is_resizing  = false,
                        is_scrolling = false,
                        parsed_event = null;

                    if( typeof event === 'string' )
                        parsed_event = event;
                    else{
                        if( typeof event === 'object' && 'type' in event )
                            parsed_event = event.type;
                        if( $.nite && undefined !== this && 'instance' in this )
                            parsed_event = this.instance;
                    }

                    if( null !== parsed_event ) switch( parsed_event ){
                        case 'scroll':
                            is_scrolling = true;
                            break;
                        case 'resize':
                            is_resizing = true;
                            break;
                        default:
                            console.error(parsed_event+' was an unknown $.fn.niteCrop() event handler.');
                            return;
                            break;
                    }

                    let store_vars = !initialized || is_resizing;

                    if( store_vars ){
                        if( $.nite ){
                            window_dimensions.width  = $.nite.width();
                            window_dimensions.height = $.nite.height();
                        }else{
                            window_dimensions.width  = $window.width();
                            window_dimensions.height = $window.height();
                        }
                    }

                    if( $.nite && store_vars )
                        $.nite.storeBoundingRects($element[0]);

                    let bn = $.extend(true, {}, $.nite ? $.nite.getBoundingRects( $element[0] ) : $element[0].getBoundingClientRect());

                    $elements.each(function (i) {

                        let _t = this;

                        if( $.nite && store_vars )
                            $.nite.storeBoundingRects(_t);

                        let $t = $(_t),
                            is_loaded = function( _this ){

                                return ( _this.complete && Math.floor(_this.naturalWidth) >= 1 && Math.floor(_this.naturalHeight) >= 1 ) || ( _this.readyState >= 2 && _this.videoWidth !== 0 && _this.videoHeight !== 0 );
                                
                            },
                            mutation_observer = function($els, callback){

                                $els.each(function() {

                                    let el = this,
                                        $el = $(el),
                                        data = $el.data(plugin_name);

                                    if ( undefined === data )
                                        return true;

                                    let mutation = data.mutationObserver;

                                    if( !mutation ){

                                        if( MutationObserver ){

                                            mutation = new MutationObserver(callback);

                                            data.mutationObserver = mutation;

                                        }

                                    }

                                    if( mutation ){

                                        let observe_attrs = [ 'id', 'class' ];

                                        // if main el observe these attributes
                                        if( $el.is($element) )
                                            observe_attrs = observe_attrs.concat([ /*'style,'*/ 'data-nite-cropper-parallax-depth' ]);

                                        // if target el observe these attributes
                                        if( $el.is($t) )
                                            observe_attrs = observe_attrs.concat([ 'src', 'srcset', 'width', 'height', 'data-width', 'data-height' ]);

                                        // user-defined attributes to observe
                                        observe_attrs = observe_attrs.concat(extra_observe_attrs);

                                        mutation.observe(el, {

                                            childList: true,

                                            attributes: true,
                                            attributeOldValue: true,
                                            attributeFilter: observe_attrs,

                                            characterData: false,
                                            characterDataOldValue: false,

                                            subtree: true

                                        });

                                    }

                                });

                            },
                            logic = function($t, i, data){

                                if( $.isNumeric(settings.width)  )
                                    data.naturalwidth  = parseFloat(settings.width);

                                if( $.isNumeric(settings.height) )
                                    data.naturalheight = parseFloat(settings.height);

                                // css3 transforms
                                let has_parallax = is_parallax(),
                                    css_matrix   = matrix($t),
                                    css_scale    = scale($t, css_matrix),
                                    css_rotation = rotation($t, css_matrix);

                                // getBoundingRect 0.xxx height fix
                                if( parsed_event === null ) {
                                    bn.width  = Math.max(bn.width, $element.width());
                                    bn.height = Math.max(bn.height, $element.height());
                                }

                                data = $.extend(true, data, nite_crop_core({ width : data.naturalwidth, height : data.naturalheight }, bn, css_scale));

                                // semplice auto-scale per garantire l'abbondanza in caso di settings.parallax
                                if( has_parallax && css_scale === 1 ) {
                                    let min_indentaton_for_parallax = 120;
                                    if( ( is_parallax('x') && Math.abs(data.indentation.x) < min_indentaton_for_parallax ) || ( is_parallax('y') && Math.abs(data.indentation.y) < min_indentaton_for_parallax ) ) {
                                        css_scale = 1.25;
                                        data = $.extend(true, data, nite_crop_core({ width : data.naturalwidth, height : data.naturalheight }, bn, css_scale));
                                    }
                                }

                                let translate_x = round_decimals(do_indent.x(bn, data, true), has_parallax ? 4 : 0 ),
                                    translate_y = round_decimals(do_indent.y(bn, data, true), has_parallax ? 4 : 0 );

                                $t
                                // set css properties
                                    .css($.extend(true, store_vars ? {
                                        maxWidth  : 'none',
                                        maxHeight : 'none',
                                        minWidth  : 0,
                                        minHeight : 0
                                    } : {}, {
                                        width : data.width,
                                        height : data.height,
                                        transform : build_transform_property({
                                            scale : css_scale,
                                            rotate : css_rotation,
                                            translate : [
                                                translate_x,
                                                translate_y
                                            ]
                                        }, has_parallax)
                                    }))
                                    // set useful data
                                    .data(plugin_name, $.extend(true, data, {
                                        rotate : css_rotation,
                                        scale  : css_scale
                                    }));

                                // trigger events
                                $element.trigger('crop.nite', [ $element, $t ]);
                                $t.trigger('crop.nite', [ $element, $t ]);

                                mutation_observer($t.add($element), logic_checks);

                            },
                            logic_checks = function(){

                                // image or video tags
                                if( ( $t.is('img') || $t.is('video') ) ){

                                    // se sono pronti
                                    if( is_loaded(_t) ) {

                                        logic($t, i, {
                                            naturalwidth  : _t.videoWidth  || _t.naturalWidth,
                                            naturalheight : _t.videoHeight || _t.naturalHeight
                                        });

                                        // se non sono pronti, aspetta che lo siano
                                    }else{

                                        $t.one( ( $t.is('video') ? 'loadedmetadata' : 'load' ) + '.' + instance, function(){
                                            logic($t, i, {
                                                naturalwidth  : _t.videoWidth  || _t.naturalWidth,
                                                naturalheight : _t.videoHeight || _t.naturalHeight
                                            });
                                        });

                                    }

                                    // other tags
                                }else{

                                    // requirements
                                    if (
                                        ( !$.isNumeric($t.attr('width')) || !$.isNumeric($t.attr('height')) ) &&
                                        ( !$.isNumeric($t.data('width')) || !$.isNumeric($t.data('height')) ) &&
                                        ( !$.isNumeric(settings.width)   || !$.isNumeric(settings.height) )
                                    )
                                        return true;

                                    let data = {};
                                    data.naturalwidth  = parseInt( $t.attr('width')  || $t.data('width') );
                                    data.naturalheight = parseInt( $t.attr('height') || $t.data('height') );
                                    data.naturalwidth  = data.naturalwidth  || settings.width;
                                    data.naturalheight = data.naturalheight || settings.height;

                                    logic($t, i, data);

                                }

                            };

                        logic_checks();

                    });

                    initialized = true;

                },
                register_window_events = function(){

                    let handler = function(cb, options){
                        return $.nite ? $.nite.event(cb, options) : function(){
                            cb.apply(this, arguments);
                        };
                    };

                    $window
                        .on('resize.'+instance, handler(initialize, { fps : 15 }))
                        .on('scroll.'+instance, handler(scroll_handler, { fps : 60 }));

                },

                deregister_window_events = function(){

                    // events
                    $window.off('.' + instance);

                };
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~


            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
            // re-call
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
            if( initialized ){

                // get stored vars
                let stored_data = $element.data(plugin_name);
                instance = stored_data.instance;
                $elements = stored_data.$elements;
                settings = stored_data.settings;

                // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                // APIs
                // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                if( typeof options === 'string' ){

                    switch( options ){

                        case 'destroy':

                            $element
                                .add($elements)
                                .removeData(plugin_name);

                            $elements
                                .css({
                                    width     : '',
                                    height    : '',
                                    transform : '',
                                    maxWidth  : '',
                                    maxHeight : '',
                                    minWidth  : '',
                                    minHeight : ''
                                });

                            deregister_window_events();

                            break;

                        case 'resize':

                            $elements = settings.elements ? $element.find(settings.elements) : $element.children('img, video');
                            $element.data(plugin_name).$elements = $elements;

                            initialize('resize');

                            deregister_window_events();

                            register_window_events();

                            break;

                        default:
                            console.error(options+' was an unknown $.fn.niteCrop() command.');
                            break;

                    }

                }

                return true; // continue this.each loop

            }
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~


            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
            if( typeof options === 'string' ) {

                console.error('$.fn.niteCrop wasn\'t initialized, you can\'t call method ' + options + ' yet.');

                return true;

            }
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~


            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
            // initialization
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
            // local vars
            settings = typeof options === 'object' ? $.extend(true, settings, options) : settings;
            instance = plugin_name + Math.floor( Math.random() * 99999 );
            $elements = settings.elements ? $element.find(settings.elements) : $element.children('img, video');
            // plugin can actually work
            if( $elements.length ){

                // useful data
                $element.data(plugin_name, {
                    $element : $element,
                    $elements : $elements,
                    instance : instance,
                    settings : settings
                });

                //
                let settings_mutation_attrs = settings.mutationObserverAttributes;
                if( typeof settings_mutation_attrs === 'string' )
                    extra_observe_attrs.push(settings_mutation_attrs);
                else if( $.isArray(settings_mutation_attrs) )
                    for( let x in settings_mutation_attrs ) {
                        let settings_mutation_attr = settings_mutation_attrs[x];
                        if (typeof settings_mutation_attr === 'string')
                            extra_observe_attrs.push(settings_mutation_attr);
                    }

                // first initalization
                initialize();

                register_window_events();

            }else
                console.warn('$.fn.niteCropper() was not able to find any element'+( settings.elements ? ' for user-provided selector "'+settings.elements+'".' : '.'));
            // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~


        });

    };
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~


    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
    // todo refactory etc
    $.fn.niteContain = function(options){

        let is_command = typeof options === 'string',
            settings = $.extend(true, { elements : null }, typeof options === 'object' ? options : {});

        return this.each(function () {

            let $t = $(this),
                $elements = typeof settings.elements === 'string' ? $t.find(settings.elements) : ( $t.is('img') ? $t : $t.children('img') );

            if( !$elements.length )
                return true;

            $elements.each(function() {

                let _this = this,
                    $this = $(_this),
                    instance = $this.data('niteContain'),
                    logic = function () {

                        let $p = $this.parent(),
                            w = $p.width(),
                            h = $p.height();

                        $this.css(_this.naturalWidth >= _this.naturalHeight ? {
                            width: w,
                            height: Math.round(_this.naturalHeight * ( w / _this.naturalWidth )),
                            maxWidth: w,
                            maxHeight: 'none'
                        } : {
                            width: Math.round(_this.naturalWidth * ( h / _this.naturalHeight )),
                            height: h,
                            maxWidth: 'none',
                            maxHeight: h
                        });

                    };

                if( is_command && instance && options === 'resize' )
                    logic();

                if( is_command && instance && options === 'destroy' ){
                    $this.removeData('niteContain').css({
                        width: '',
                        height: '',
                        maxWidth: '',
                        maxHeight: ''
                    });
                    $window.off('resize.'+instance);
                }

                if( !is_command && !instance ) {

                    let instance = $.nite.uniqueId();

                    $this.data('niteContain', instance);

                    $window.on('resize.'+instance, $.nite.event(function () {

                        if (_this.complete && Math.floor(_this.naturalWidth) >= 1 && Math.floor(_this.naturalHeight) >= 1)
                            logic();
                        else
                            $this.one('load', logic);

                    }, { selfExecuting: true, fps: 8 }));

                }

            });

        });
    };
    // ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~


})(window, document, jQuery);
// ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~